
def to_mst(f: float):
    """
    Convert float input to minutes/seconds/thirds/fourths
    """
    _r = f * 60 * 60 * 60 * 60
    minutes, _r = divmod(_r, 60*60*60*60)
    seconds, _r = divmod(_r, 60*60*60)
    thirds, _r  = divmod(_r, 60*60)
    fourths, _r = divmod(_r, 60)
    return int(minutes), int(seconds), int(thirds), int(fourths)

def to_float(m,*args):
    t = m 
    d = 60
    for a in args:
        t += a/d
        d *= 60
    return t


# Class that implements Minutes, seconds, thirds, fourths
class MST(object):
    def __init__(self, minutes, *rest):
        if rest:
            self.mst = (minutes, *rest)
        else:
            self.mst = to_mst(minutes)

    def __add__(self, other):
        a = to_mst(float(self) + float(other))
        return MST(*a)

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        a = to_mst(float(self) - float(other))
        return MST(*a)

    def __rsub__(self, other):
        a = to_mst(float(other) - float(self))
        return MST(*a)

    def __truediv__(self, other):
        a = to_mst(float(self) / float(other))
        return MST(*a)
    
    def __mul__(self, other):
        a = to_mst(float(self) * float(other))
        return MST(*a)

    def __rmul__(self, other):
        return self.__mul__(other);

    def __pow__(self, other):
        a = to_mst(float(self)**other)
        return MST(*a)
       
    def __float__(self):
        return to_float(*self.mst)

    def __str__(self):
        return ",".join([str(x) for x in self.mst])

    def __repr__(self):
        return str(self)

    



import numpy as np
from fractions import Fraction 

_k = [0 for i in range(24)]
_b = [0 for i in range(24)]

_k[0] = _b[0] = 224 + Fraction(50, 60) + Fraction(22, 3600)

# raNgebAlAstrI
divisor = 233 + Fraction(32, 60)

# Nilakantha recurrence relations
for i in range(1,24):
    _k[i] = _k[i-1] - (_b[i-1] / divisor)
    _b[i] = _b[i-1] + _k[i]

def khandajya(k: int):
    """
    Return the kth khandajya computed by the nilakantha recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth khandajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return float(_k[k])

def pindajya(k: int):
    """
    Return the kth pindajya computed by the nilakantha recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth pindajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return float(_b[k])



import numpy as np
from fractions import Fraction 
from kerala_math.jyatable import MST

_k = [0 for i in range(24)]
_b = [0 for i in range(24)]

# First two accurate jyas
_b[0] = MST(224, 50, 22) 
_b[1] = MST(448, 42, 58)

# tattajya recurrence
for i in range(2,24):
    _b[i] = ((_b[i-1])**2 - (_b[0])**2)/_b[i-2] 

# Pindajyas
_k[0] = _b[0]
for i in range(1,24):
    _k[i] = _b[i] - _b[i-1]

    
def khandajya(k: int):
    """
    Return the kth khandajya computed by the tattajya recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth khandajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _k[k]

def pindajya(k: int):
    """
    Return the kth pindajya computed by the tattajya recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth pindajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _b[k]



#! /usr/bin/env python
#
#  Various series for circumference derived by the kerala school
#
import numpy as np
from fractions import Fraction 

# The vyAse vAridhinihate series
def vyase(v, n=10, samskara=3, calc_type="fraction"):
    '''
    Calculates the paridhi (circumference) from the vyAsa (diameter)

    Uses the famous Madhava series

    व्यासे वारिधिनिहते रूपहृते व्याससागराभिहते ।
    त्रिशरादिविषमसंख्याभक्तमृणं स्वं पृथक्क्रमात् कुर्यात् ॥

    Inputs:
                   v: diameter
                   n: number of terms
      samskara: None (default) or
                      k for kth order end-correction
                      where k in (0, 1, 2, 3)
           calc_type: "fraction" (default): uses fractions
                      "float": uses numpy float128
    Returns:
                   p: circumference
    '''
    assert ((calc_type == "fraction") or (calc_type == "float")), \
        f"Unknown calc_type {calc_type}: pass 'float' or 'fraction'"
    if calc_type == "fraction":
        t = Fraction(4*v, 1) # First term
    else:
        t = np.float128(4*v)  # First term
    # Initialize the series to first term
    s = t     
    sg = -1   # Sign of next term
    # C = 4v - 4v/3 + 4v/5 - 4v/7 ...
    for i in range(2, n+1):   # Ends after the nth term
        s += sg*t/(2*i - 1)    # 4v/(2n-1) is the last term
        sg *= -1
    # Add end term
    # Last divisor
    if samskara == 0:
        # Zeroth order correction
        # a_p = 4v/2p, where 4v/p is the last term
        p = 2*n - 1
        s += sg*t/(2*p)
    elif samskara == 1:
        # First order
        # a_p = 4v/(2p+2) = 4v/4n 
        s += sg*t/(4*n)
    elif samskara == 2:
        # Second Order
        # a_p = 4v/((2p+2) + 4/(2p+2))
        #     = 4v.n/(4n^2 + 1)
        s += sg*t*n/(4*n**2 + 1)
    elif samskara == 3:
        # Third Order
        # a_p = 4v/((2p+2) + 4/((2p+2) + 16/(2p+2)))
        #     = 4v.(n^2 + 1)/(4n^3+5n)
        s += sg*t*(n**2 + 1)/(4*n**3 + 5*n)
    return s

# The vyAsavargAdravihatAt series
def vyasavargad(v, n=10, calc_type="fraction"):
    '''
    Calculates the paridhi (circumference) from the vyAsa (diameter)

    Uses the vyAsavargAdravihatAt series
    vyāsavargād ravihatāt padam syāt prathamam phalam |
    tadāditastrisamkhyāptam phalam syāduttarottaram ||
    rupādyayugmasamkhyābhirhr.tes.ves.u yathākramam |
    vis.amānāmyutestyaktvā samam hi paridhirbhavet ||

    Inputs:
                   v: diameter
                   n: number of terms
           calc_type: "fraction" (default): uses fractions
                      "float": uses numpy float128
    Returns:
                   p: circumference
    '''
    assert ((calc_type == "fraction") or (calc_type == "float")), \
        f"Unknown calc_type {calc_type}: pass 'float' or 'fraction'"
    if calc_type == "fraction":
        # Fixme implement Aryabhata square root algo
        t = Fraction(np.sqrt(12 * v**2)) # First term
    else:
        t = np.sqrt(np.float128(12 * v**2))  # First term
    p = t
    s = -1   # Sign of next term
    # C = sqrt(12d^2) (1 - 1/3.3 + 1/(3^2.5)+1/(3^3.7) ...) 
    for i in range(1,n):
        if calc_type == "fraction":
            p += Fraction(s*t, (3**i*(2*i+1)))
        else:
            p += s*t/np.float64(3**i*(2*i+1))

        s = s*-1
    return p


def samapanchahatayoh(v, n=10, calc_type="fraction"):
    '''
    Calculates the paridhi (circumference) from the vyAsa (diameter) using 
the samapanchahatayoH series
    
    समपञ्चाहतयो या रुपाद्ययुजां चतुर्घ्नमूलयुताः 
    ताभिः षोडशगुणितात् पृथगाहृतेषु विषमयुतेः 
    समफलयुतिमपहाय स्यादिष्टव्याससंभवः परिधिः 

    Inputs:
                   v: diameter
                   n: number of terms
           calc_type: "fraction" (default): uses fractions
                      "float": uses numpy float128
    Returns:
                   p: circumference
    '''
    assert ((calc_type == "fraction") or (calc_type == "float")), \
        f"Unknown calc_type {calc_type}: pass 'float' or 'fraction'"
    sg = 1   # Sign of next term
    t = 16*v
    s = 0
    # C = 16d(1/1^5+4.1 - 1/3^5+4*3 + 1/5^5+4*5 ...) 
    for i in range(0,n):
        o = 2*i+1
        if calc_type == "fraction":
            s += Fraction(sg*t, (o**5+4*o))
        else:
            s += sg*t/np.float64(o**5+4*o)

        sg = sg*-1
    return s


def vyasad(v, n=10, calc_type="fraction"):
    '''
    Calculates the paridhi (circumference) from the vyAsa (diameter) using 
the vyAsAd vAridhinihatAt series

    व्यासाद् वारिधिनिहतात् पृथगाप्तं त्र्याद्ययुग्विमूलघनैः
    त्रिघ्नव्यासे स्वमृणं क्रमशः कृत्वा परिधिरानेयः ||  
    C = 3D + 4D(1 /(3^3-3) - 1/(5^3-5) + 1/(7^3-7) ….)

    Inputs:
                   v: diameter
                   n: number of terms
           calc_type: "fraction" (default): uses fractions
                      "float": uses numpy float128
    Returns:
                   p: circumference
    '''
    assert ((calc_type == "fraction") or (calc_type == "float")), \
        f"Unknown calc_type {calc_type}: pass 'float' or 'fraction'"
    sg = 1   # Sign of next term
    t = 4*v
    s = 3*v
    # C = 3D + 4D(1 /(3^3-3) - 1/(5^3-5) + 1/(7^3-7) ….)
    for i in range(1,n):
        o = 2*i+1
        if calc_type == "fraction":
            s += Fraction(sg*t, (o**3-o))
        else:
            s += sg*t/np.float64(o**3-o)
        sg = sg*-1
    return s


__all__ = ["vyase", "vyasavargad", "samapanchahatayoh", "vyasad"]



#! /usr/bin/env python
#
#  Various series for jya / sara etc derived by the kerala school
#


import numpy as np
from kerala_math.jyatable import MST

# srIrudraH sRIDaraH shresTho devo vishvasthalI bhrguH
R = MST(3437, 44, 48, 22, 29, 22, 22)
#  devo vishvasthalI bhrguH
#R = MST(3437, 44, 48, 42)
_Rf = float(R)

# Nihatya cApavargeNa ... 
def jya_nihatya(s, terms=10, mst_out=False):
    j = s # Zeroth Approximation
    t = j # Previous Term 
    sg = -1 # Next sign
    m = s**2 / (_Rf**2) # Multiplier
    for i in range(1, terms):
        t *= m / (2*i + (2*i)**2)
        j += sg*t
        sg *= -1
    if mst_out:
        return MST(j)
    else:
        return j


# Nihatya cApavargeNa ... 
def sara_nihatya(s, terms=10, mst_out=False):
    j = s**2/(2*_Rf) # Zeroth Approximation
    t = j # Previous Term 
    sg = -1 # Next sign
    m = s**2 / (_Rf**2) # Multiplier
    for i in range(1, terms):
        t *= m/((2*(i+1))**2-2*(i+1))
        j += sg*t
        sg *= -1
    if mst_out:
        return MST(j)
    else:
        return j

# vidvAnstunnabalaH
def jya_vidvan(s, mst_out=False):
    vt = [
        MST(0,0,44), MST(0, 33, 6), MST(16,5,41),
        MST(273, 57, 47), MST(2220, 39, 40)
    ]
    # Square of 5400 = नानाज्ञानतपोधरः
    q2 = 29160000
    # cube of 5400 = अज्ञाननुन्ने नवतत्त्वसंशयः
    q3 = 157464000000
    s2 = s**2
    s3 = s**3
    m2 = s2/q2
    m3 = s3/q3
    t = 0
    for v in vt:
        t = float(v) - m2*t
    j = s - m3*t
    if mst_out:
        return MST(j)
    else:
        return j
   

# stenastrI
def sara_stena(s, mst_out=False):
    vt = [
        MST(0,0,6), MST(0, 5, 12), MST(3, 9, 37), MST(71, 43, 24),
        MST(872, 3, 5), MST(4241, 9, 0)
    ]
    # Square of 5400 = नानाज्ञानतपोधरः
    q2 = 29160000
    s2 = s**2
    m2 = s2/q2
    t = 0
    for v in vt:
        t = m2*(float(v) - t)
    j = t
    if mst_out:
        return MST(j)
    else:
        return j
    



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding

from os import path
import codecs

def read(rel_path):
    here = path.abspath(path.dirname(__file__))
    with codecs.open(path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
long_description = ''
try:
    import pypandoc

    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = ''

setup(
  name='kerala_math',

  # Versions should comply with PEP440.  For a discussion on single-sourcing
  # the version across setup.py and the project code, see
  # https://packaging.python.org/en/latest/single_source_version.html
  # version='0.2.2-post0',
    version=get_version("kerala_math/__init__.py"),
  description='Mathematical methods from the Kerala School of Mathematics',
  long_description=long_description,

  # The project's main homepage.
  url='https://github.com/kmadathil/kerala_math',

  # Author details
  author='Karthik Madathil',
  author_email='karthik@madathil.org',

  # Choose your license
  license='MIT',

  # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
  classifiers=[
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 3 - Alpha',

    # Indicate who your project is intended for
    'Intended Audience :: Education',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
    # Left commented as this category doesn't exist
    # 'Natural Language :: Sanskrit',

    # Pick your license as you wish (should match "license" above)
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
  ],

  # What does your project relate to?
  keywords='math mathematics kerala sanskrit samskritam',

  # You can just specify the packages manually here if your project is
  # simple. Or you can use find_packages().
  packages=find_packages(exclude=['contrib', 'docs', 'tests', 'metrics']),

  # Alternatively, if you want to distribute just a my_module.py, uncomment
  # this:
  #   py_modules=["my_module"],

  # List run-time dependencies here.  These will be installed by pip when
  # your project is installed. For an analysis of "install_requires" vs pip's
  # requirements files see:
  # https://packaging.python.org/en/latest/requirements.html
  install_requires=[],

  # List additional groups of dependencies here (e.g. development
  # dependencies). You can install these using the following syntax,
  # for example:
  # $ pip install -e .[dev,test]
  extras_require={
  #     'dev': ['check-manifest'],
  #     'test': ['pandas', 'xlrd'],
  },

  # If there are data files included in your packages that need to be
  # installed, specify them here.  If using Python 2.6 or less, then these
  # have to be included in MANIFEST.in as well.
  package_data={
  },

  # Although 'package_data' is the preferred approach, in some case you may
  # need to place data files outside of your packages. See:
  # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
  # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
  # data_files=[('my_data', ['data/data_file'])],

  # scripts = ["scripts/sanskrit_parser"],
  # To provide executable scripts, use entry points in preference to the
  # "scripts" keyword. Entry points provide cross-platform support and allow
  # pip to create the appropriate form of executable for the target platform.
  # entry_points={
  #     'console_scripts': [
  #       ],
  # },
)



import numpy as np
from fractions import Fraction 

_b = [0 for i in range(24)]

#_k[0] = _b[0] = 225

# # Aryabhata recurrence relations
# for i in range(1,24):
#     _k[i] = _k[i-1] - _b[i-1]/_b[0]
#     _b[i] = _b[i-1] + _k[i]

# khandajyas maKi baKi etc
_k = [225, 224, 222, 219, 215, 210, 205, 199, 191, 183, 174, 164, \
      154, 143, 131, 119, 106,  93,  79,  65,  51,  37,  22,   7]

# Pindajyas
_b[0] = 225
for i in range(1,24):
    _b[i] = _b[i-1] + _k[i]

def khandajya(k: int):
    """
    Return the kth khandajya computed by the aryabhata recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth khandajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _k[k]

def pindajya(k: int):
    """
    Return the kth pindajya computed by the aryabhata recurrence

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth pindajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _b[k]



import numpy as np
from fractions import Fraction 
from kerala_math.jyatable import MST

_k = [0 for i in range(24)]

# śreṣṭhaṁ nāma variṣṭhānāṁ 
_b = [
    MST(224, 50, 22), # श्रेष्ठं नाम वरिष्ठानां
    MST(448, 42, 58), # हिमाद्रिर्वेदभावनः
    MST(670, 40, 16), # तपनो भानु सूक्तज्ञो
    MST(889, 45, 15), # मध्यमं विद्धि दोहनं
    MST(1105, 1, 39), # धिगाज्योनाशनं कष्टं
    MST(1315, 34,  7), # छन्नभोगाशयाम्बिका
    MST(1520, 28, 35), # मृगाहारो नरेशोयं
    MST(1718, 52, 24), # वीरो रणजयोत्सुकः
    MST(1909, 54, 35), # मूलं विशुद्धं नाळस्य
    MST(2092, 46,  3), # गानेषु विरळा नराः
    MST(2266, 39, 50), # अशुद्धिगुप्ता चोरश्रीः
    MST(2430, 51, 15), # शङ्कुकर्णो नगेश्वरः
    MST(2584, 38,  6), # तनुजो गर्भजो मित्रं
    MST(2727, 20, 52), # श्रीमानत्र सुखी सखे
    MST(2858, 22, 55), # शशी रात्रौ हिमाहारौ
    MST(2977, 10, 34), # वेगज्ञः पथि सिन्धुरः
    MST(3083, 13, 17), # छायालयो गजो नीलो
    MST(3176,  3, 50), # निर्मलो नास्ति सत्कुले
    MST(3255, 18, 22), # रात्रौ दर्पणमभ्राङ्गं
    MST(3320, 36, 30), # नागस्तुङ्गनखो बली
    MST(3371, 41, 29), # धीरो युवा कथालोलः
    MST(3408, 20, 11), # पूज्यो नारीजनैर्भगः
    MST(3430, 23, 11), # कन्यागारे नागवल्ली
    MST(3437, 44, 48)  # देवो विश्वस्थली भृगुः 
]

# Pindajyas
_k[0] = _b[0]
for i in range(1,24):
    _k[i] = _b[i] - _b[i-1]

madhava_verse =  [
    "श्रेष्ठं नाम वरिष्ठानां",
    "हिमाद्रिर्वेदभावनः",
    "तपनो भानु सूक्तज्ञो",
    "मध्यमं विद्धि दोहनं",
    "धिगाज्योनाशनं कष्टं",
     "छन्नभोगाशयाम्बिका",
     "मृगाहारो नरेशोयं",
     "वीरो रणजयोत्सुकः",
     "मूलं विशुद्धं नाळस्य",
     "गानेषु विरळा नराः",
     "अशुद्धिगुप्ता चोरश्रीः",
     "शङ्कुकर्णो नगेश्वरः",
     "तनुजो गर्भजो मित्रं",
     "श्रीमानत्र सुखी सखे",
     "शशी रात्रौ हिमाहारौ",
     "वेगज्ञः पथि सिन्धुरः",
     "छायालयो गजो नीलो",
     "निर्मलो नास्ति सत्कुले",
     "रात्रौ दर्पणमभ्राङ्गं",
     "नागस्तुङ्गनखो बली",
     "धीरो युवा कथालोलः",
     "पूज्यो नारीजनैर्भगः",
     "कन्यागारे नागवल्ली",
     "देवो विश्वस्थली भृगुः", 
]

    
    
def khandajya(k: int):
    """
    Return the kth khandajya computed by the madhava table

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth khandajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _k[k]

def pindajya(k: int):
    """
    Return the kth pindajya computed by the madhava table

    Inputs:
      k (int) : index from 0 to 23
    Outputs:
      int : kth pindajya
    """
    assert ((k>=0) & (k<24)), f"Input must be between 0 and 23 inclusive, got {k}"
    return _b[k]
